#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const workspace_1 = require("@nrwl/workspace");
const new_1 = require("@nrwl/workspace/src/schematics/new/new");
const package_manager_1 = require("@nrwl/tao/src/shared/package-manager");
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const inquirer = require("inquirer");
const path = require("path");
const tmp_1 = require("tmp");
const yargsParser = require("yargs-parser");
const shared_1 = require("./shared");
const presetOptions = [
    {
        value: new_1.Preset.Empty,
        name: 'empty             [an empty workspace with a layout that works best for building apps]',
    },
    {
        value: new_1.Preset.React,
        name: 'react             [a workspace with a single React application]',
    },
    {
        value: new_1.Preset.Angular,
        name: 'angular           [a workspace with a single Angular application]',
    },
    {
        value: new_1.Preset.NextJs,
        name: 'next.js           [a workspace with a single Next.js application]',
    },
    {
        value: new_1.Preset.Nest,
        name: 'nest              [a workspace with a single Nest application]',
    },
    {
        value: new_1.Preset.WebComponents,
        name: 'web components    [a workspace with a single app built using web components]',
    },
    {
        value: new_1.Preset.ReactWithExpress,
        name: 'react-express     [a workspace with a full stack application (React + Express)]',
    },
    {
        value: new_1.Preset.AngularWithNest,
        name: 'angular-nest      [a workspace with a full stack application (Angular + Nest)]',
    },
    {
        value: new_1.Preset.OSS,
        name: 'oss               [an empty workspace with a layout that works best for open-source projects]',
    },
];
const tsVersion = '4.0.5';
const cliVersion = '11.0.19';
const nxVersion = '11.0.19';
const prettierVersion = '2.1.2';
const parsedArgs = yargsParser(process.argv.slice(2), {
    string: [
        'cli',
        'preset',
        'appName',
        'style',
        'linter',
        'defaultBase',
        'packageManager',
    ],
    alias: {
        packageManager: 'pm',
    },
    boolean: ['help', 'interactive', 'nxCloud'],
    default: {
        interactive: false,
    },
    configuration: {
        'strip-dashed': true,
        'strip-aliased': true,
    },
});
if (parsedArgs.help) {
    showHelp();
    process.exit(0);
}
const packageManager = parsedArgs.packageManager || 'npm';
determineWorkspaceName(parsedArgs).then((name) => {
    determinePreset(parsedArgs).then((preset) => {
        return determineAppName(preset, parsedArgs).then((appName) => {
            return determineStyle(preset, parsedArgs).then((style) => {
                return determineCli(preset, parsedArgs).then((cli) => {
                    return determineLinter(preset, parsedArgs).then((linter) => {
                        return askAboutNxCloud(parsedArgs).then((nxCloud) => {
                            const tmpDir = createSandbox(packageManager);
                            createApp(tmpDir, name, Object.assign(Object.assign({}, parsedArgs), { cli,
                                preset,
                                appName,
                                style,
                                linter,
                                nxCloud }));
                            shared_1.showNxWarning(name);
                            pointToTutorialAndCourse(preset);
                        });
                    });
                });
            });
        });
    });
});
function showHelp() {
    const options = Object.values(new_1.Preset)
        .map((preset) => '"' + preset + '"')
        .join(', ');
    console.log(`
  Usage: create-nx-workspace <name> [options] [new workspace options]

  Create a new Nx workspace

  Options:

    name                      Workspace name (e.g., org name)

    preset                    What to create in a new workspace (options: ${options})

    appName                   The name of the application created by some presets  

    cli                       CLI to power the Nx workspace (options: "nx", "angular")
    
    style                     Default style option to be used when a non-empty preset is selected 
                              options: ("css", "scss", "styl", "less") for React/Next.js also ("styled-components", "@emotion/styled")    
                              
    linter                    Default linter. Options: "eslint", "tslint".

    interactive               Enable interactive mode when using presets (boolean)
    
    nx-cloud                  Use Nx Cloud (boolean)

    [new workspace options]   any 'new workspace' options
`);
}
function determineWorkspaceName(parsedArgs) {
    const workspaceName = parsedArgs._[0];
    if (workspaceName) {
        return Promise.resolve(workspaceName);
    }
    return inquirer
        .prompt([
        {
            name: 'WorkspaceName',
            message: `Workspace name (e.g., org name)    `,
            type: 'string',
        },
    ])
        .then((a) => {
        if (!a.WorkspaceName) {
            workspace_1.output.error({
                title: 'Invalid workspace name',
                bodyLines: [`Workspace name cannot be empty`],
            });
            process.exit(1);
        }
        return a.WorkspaceName;
    });
}
function determinePreset(parsedArgs) {
    if (parsedArgs.preset) {
        if (Object.values(new_1.Preset).indexOf(parsedArgs.preset) === -1) {
            workspace_1.output.error({
                title: 'Invalid preset',
                bodyLines: [
                    `It must be one of the following:`,
                    '',
                    ...Object.values(new_1.Preset),
                ],
            });
            process.exit(1);
        }
        else {
            return Promise.resolve(parsedArgs.preset);
        }
    }
    return inquirer
        .prompt([
        {
            name: 'Preset',
            message: `What to create in the new workspace`,
            default: 'empty',
            type: 'list',
            choices: presetOptions,
        },
    ])
        .then((a) => a.Preset);
}
function determineAppName(preset, parsedArgs) {
    if (preset === new_1.Preset.Empty || preset === new_1.Preset.OSS) {
        return Promise.resolve('');
    }
    if (parsedArgs.appName) {
        return Promise.resolve(parsedArgs.appName);
    }
    return inquirer
        .prompt([
        {
            name: 'AppName',
            message: `Application name                   `,
            type: 'string',
        },
    ])
        .then((a) => {
        if (!a.AppName) {
            workspace_1.output.error({
                title: 'Invalid name',
                bodyLines: [`Name cannot be empty`],
            });
            process.exit(1);
        }
        return a.AppName;
    });
}
function determineCli(preset, parsedArgs) {
    if (parsedArgs.cli) {
        if (['nx', 'angular'].indexOf(parsedArgs.cli) === -1) {
            workspace_1.output.error({
                title: 'Invalid cli',
                bodyLines: [`It must be one of the following:`, '', 'nx', 'angular'],
            });
            process.exit(1);
        }
        return Promise.resolve(parsedArgs.cli);
    }
    switch (preset) {
        case new_1.Preset.Angular:
        case new_1.Preset.AngularWithNest: {
            return Promise.resolve('angular');
        }
        default: {
            return Promise.resolve('nx');
        }
    }
}
function determineStyle(preset, parsedArgs) {
    if (preset === new_1.Preset.Empty ||
        preset === new_1.Preset.OSS ||
        preset === new_1.Preset.Nest) {
        return Promise.resolve(null);
    }
    const choices = [
        {
            value: 'css',
            name: 'CSS',
        },
        {
            value: 'scss',
            name: 'SASS(.scss)  [ http://sass-lang.com   ]',
        },
        {
            value: 'styl',
            name: 'Stylus(.styl)[ http://stylus-lang.com ]',
        },
        {
            value: 'less',
            name: 'LESS         [ http://lesscss.org     ]',
        },
    ];
    if ([new_1.Preset.ReactWithExpress, new_1.Preset.React, new_1.Preset.NextJs].includes(preset)) {
        choices.push({
            value: 'styled-components',
            name: 'styled-components [ https://styled-components.com            ]',
        }, {
            value: '@emotion/styled',
            name: 'emotion           [ https://emotion.sh                       ]',
        }, {
            value: 'styled-jsx',
            name: 'styled-jsx        [ https://www.npmjs.com/package/styled-jsx ]',
        });
    }
    if (!parsedArgs.style) {
        return inquirer
            .prompt([
            {
                name: 'style',
                message: `Default stylesheet format          `,
                default: 'css',
                type: 'list',
                choices,
            },
        ])
            .then((a) => a.style);
    }
    const foundStyle = choices.find((choice) => choice.value === parsedArgs.style);
    if (foundStyle === undefined) {
        workspace_1.output.error({
            title: 'Invalid style',
            bodyLines: [
                `It must be one of the following:`,
                '',
                ...choices.map((choice) => choice.value),
            ],
        });
        process.exit(1);
    }
    return Promise.resolve(parsedArgs.style);
}
function determineLinter(preset, parsedArgs) {
    if (!parsedArgs.linter) {
        if (preset === new_1.Preset.Angular || preset === new_1.Preset.AngularWithNest) {
            return inquirer
                .prompt([
                {
                    name: 'linter',
                    message: `Default linter                     `,
                    default: 'tslint',
                    type: 'list',
                    choices: [
                        {
                            value: 'tslint',
                            name: 'TSLint [ Used by Angular CLI ]',
                        },
                        {
                            value: 'eslint',
                            name: 'ESLint [ Modern linting tool ]',
                        },
                    ],
                },
            ])
                .then((a) => a.linter);
        }
        else {
            return Promise.resolve('eslint');
        }
    }
    else {
        if (parsedArgs.linter !== 'eslint' && parsedArgs.linter !== 'tslint') {
            workspace_1.output.error({
                title: 'Invalid linter',
                bodyLines: [`It must be one of the following:`, '', 'eslint', 'tslint'],
            });
            process.exit(1);
        }
        else {
            return Promise.resolve(parsedArgs.linter);
        }
    }
}
function createSandbox(packageManager) {
    console.log(`Creating a sandbox with Nx...`);
    const tmpDir = tmp_1.dirSync().name;
    fs_1.writeFileSync(path.join(tmpDir, 'package.json'), JSON.stringify({
        dependencies: {
            '@nrwl/workspace': nxVersion,
            '@nrwl/tao': cliVersion,
            typescript: tsVersion,
            prettier: prettierVersion,
        },
        license: 'MIT',
    }));
    child_process_1.execSync(`${packageManager} install --silent`, {
        cwd: tmpDir,
        stdio: [0, 1, 2],
    });
    return tmpDir;
}
function createApp(tmpDir, name, parsedArgs) {
    const { _, cli } = parsedArgs, restArgs = tslib_1.__rest(parsedArgs, ["_", "cli"]);
    const args = workspace_1.unparse(restArgs).join(' ');
    const pmc = package_manager_1.getPackageManagerCommand(packageManager);
    const command = `new ${name} ${args} --collection=@nrwl/workspace`;
    console.log(command);
    const nxWorkspaceRoot = process.cwd().replace(/\\/g, '/');
    child_process_1.execSync(`${pmc.exec} tao ${command}/collection.json --cli=${cli} --nxWorkspaceRoot="${nxWorkspaceRoot}"`, {
        stdio: [0, 1, 2],
        cwd: tmpDir,
    });
    if (parsedArgs.nxCloud) {
        workspace_1.output.addVerticalSeparator();
        child_process_1.execSync(`${pmc.exec} nx g @nrwl/nx-cloud:init --no-analytics`, {
            stdio: [0, 1, 2],
            cwd: path.join(process.cwd(), name),
        });
    }
}
function askAboutNxCloud(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.nxCloud === undefined) {
            return inquirer
                .prompt([
                {
                    name: 'NxCloud',
                    message: `Use Nx Cloud? (It's free and doesn't require registration.)`,
                    type: 'list',
                    choices: [
                        {
                            value: 'yes',
                            name: 'Yes [Faster builds, run details, Github integration. Learn more at https://nx.app]',
                        },
                        {
                            value: 'no',
                            name: 'No',
                        },
                    ],
                    default: 'no',
                },
            ])
                .then((a) => a.NxCloud === 'yes');
        }
        else {
            return parsedArgs.nxCloud;
        }
    });
}
function pointToTutorialAndCourse(preset) {
    const title = `First time using Nx? Check out this interactive Nx tutorial.`;
    switch (preset) {
        case new_1.Preset.React:
        case new_1.Preset.ReactWithExpress:
        case new_1.Preset.NextJs:
            workspace_1.output.addVerticalSeparator();
            workspace_1.output.note({
                title: title,
                bodyLines: [
                    `https://nx.dev/react/tutorial/01-create-application`,
                    ...pointToFreeCourseOnEgghead(),
                ],
            });
            break;
        case new_1.Preset.Angular:
        case new_1.Preset.AngularWithNest:
            workspace_1.output.addVerticalSeparator();
            workspace_1.output.note({
                title: title,
                bodyLines: [
                    `https://nx.dev/angular/tutorial/01-create-application`,
                    ...pointToFreeCourseOnYoutube(),
                ],
            });
            break;
        case new_1.Preset.Nest:
            workspace_1.output.addVerticalSeparator();
            workspace_1.output.note({
                title,
                bodyLines: [
                    `https://nx.dev/node/tutorial/01-create-application`,
                    ...pointToFreeCourseOnYoutube(),
                ],
            });
            break;
    }
}
function pointToFreeCourseOnYoutube() {
    return [
        ``,
        `Prefer watching videos? Check out this free Nx course on YouTube.`,
        `https://www.youtube.com/watch?v=2mYLe9Kp9VM&list=PLakNactNC1dH38AfqmwabvOszDmKriGco`,
    ];
}
function pointToFreeCourseOnEgghead() {
    return [
        ``,
        `Prefer watching videos? Check out this free Nx course on Egghead.io.`,
        `https://egghead.io/playlists/scale-react-development-with-nx-4038`,
    ];
}
//# sourceMappingURL=create-nx-workspace.js.map